@@ -71,6 +71,17 @@ def url_norm (url, encoding=None):
         raise LinkCheckerError(msg)
 
 
+def getXmlText (parent, tag):
+    """Return XML content of given tag in parent element."""
+    elem = parent.getElementsByTagName(tag)[0]
+    # Yes, the DOM standard is awful.
+    rc = []
+    for node in elem.childNodes:
+        if node.nodeType == node.TEXT_NODE:
+            rc.append(node.data)
+    return ''.join(rc)
+
+
 class UrlBase (object):
     """An URL with additional information like validity etc."""
 
@@ -310,7 +321,7 @@ class UrlBase (object):
         self.anchors = cache_data["anchors"]
         self.content_type = cache_data["content_type"]
         self.cached = True
-        if anchor_changed and self.valid and self.anchor:
+        if anchor_changed and self.valid:
             # recheck anchor
             if self.check_anchor():
                 # a new warning has been added - remove cached flag
@@ -433,8 +444,8 @@ class UrlBase (object):
             return
         # check for obfuscated IP address
         if iputil.is_obfuscated_ip(self.host):
-             ips = iputil.resolve_host(self.host)
-             if ips:
+            ips = iputil.resolve_host(self.host)
+            if ips:
                 self.add_warning(
                    _("URL %(url)s has obfuscated IP address %(ip)s") % \
                    {"url": self.base_url, "ip": ips.pop()},
@@ -603,7 +614,7 @@ class UrlBase (object):
         return handler.follow
 
     def get_anchors (self):
-        """Store list of anchors for this URL. Precondition: this URL is
+        """Store anchors for this URL. Precondition: this URL is
         an HTML resource."""
         log.debug(LOG_CHECK, "Getting HTML anchors %s", self)
         handler = linkparse.LinkFinder(self.add_anchor,
@@ -625,15 +636,17 @@ class UrlBase (object):
         self.anchors.append((url, line, column, name, base))
 
     def check_anchor (self):
-        """If URL was valid and has an anchor, check it. A warning is
-        logged and True is returned if the anchor is not found.
+        """If URL is valid, parseable and has an anchor, check it.
+        A warning is logged and True is returned if the anchor is not found.
         """
-        if not self.aggregate.config["anchors"]:
+        if not (self.anchor and self.aggregate.config["anchors"] and
+                self.valid and self.is_html()):
             return
         log.debug(LOG_CHECK, "checking anchor %r in %s", self.anchor, self.anchors)
         if any(x for x in self.anchors if x[0] == self.anchor):
             return
-        anchors = u",".join(u"`%s'" % x[0] for x in self.anchors)
+        anchornames = sorted(set(u"`%s'" % x[0] for x in self.anchors))
+        anchors = u", ".join(anchornames)
         args = {"name": self.anchor, "anchors": anchors}
         msg = u"%s %s" % (_("Anchor `%(name)s' not found.") % args,
                           _("Available anchors: %(anchors)s.") % args)
@@ -702,8 +715,7 @@ class UrlBase (object):
             self.set_title_from_content()
             if self.aggregate.config["anchors"]:
                 self.get_anchors()
-        if self.anchor:
-            self.check_anchor()
+        self.check_anchor()
         self.check_warningregex()
         # is it an intern URL?
         if not self.extern[0]:
@@ -722,17 +734,31 @@ class UrlBase (object):
 
     def check_warningregex (self):
         """Check if content matches a given regular expression."""
-        warningregex = self.aggregate.config["warningregex"]
-        if warningregex:
-            log.debug(LOG_CHECK, "checking content")
-            try:
-                match = warningregex.search(self.get_content())
-                if match:
-                    self.add_warning(_("Found %(match)r in link contents.") %
-                       {"match": match.group()}, tag=WARN_URL_WARNREGEX_FOUND)
-            except tuple(ExcList):
-                value = self.handle_exception()
-                self.set_result(unicode_safe(value), valid=False)
+        config = self.aggregate.config
+        warningregex = config["warningregex"]
+        if not (warningregex and self.valid and self.is_parseable()):
+            return
+        log.debug(LOG_CHECK, "checking content for warning regex")
+        try:
+            content = self.get_content()
+            curpos = 0
+            curline = 1
+            # add warnings for found matches, up to the maximum allowed number
+            for num, match in enumerate(warningregex.finditer(content)):
+                # calculate line number for match
+                curline += content.count('\n', curpos, match.start())
+                curpos = match.start()
+                # add a warning message
+                msg = _("Found %(match)r at line %(line)d in link contents.")
+                self.add_warning(msg %
+                   {"match": match.group(), "line": curline},
+                   tag=WARN_URL_WARNREGEX_FOUND)
+                # check for maximum number of warnings
+                if num >= config["warningregex_max"]:
+                    break
+        except tuple(ExcList):
+            value = self.handle_exception()
+            self.set_result(unicode_safe(value), valid=False)
 
     def check_size (self):
         """Check content size if it is zero or larger than a given
@@ -814,6 +840,21 @@ class UrlBase (object):
                 _("warning: cssutils parsing caused error: %(msg)s") %
                 {"msg": err})
 
+    def check_w3_errors (self, xml, w3type):
+        """Add warnings for W3C HTML or CSS errors in xml format.
+        w3type is either "W3C HTML" or "W3C CSS"."""
+        from xml.dom.minidom import parseString
+        dom = parseString(xml)
+        for error in dom.getElementsByTagName('m:error'):
+            warnmsg = _("%(w3type)s validation error at line %(line)s col %(column)s: %(msg)s")
+            attrs = {
+                "w3type": w3type,
+                "line": getXmlText(error, "m:line"),
+                "column": getXmlText(error, "m:col"),
+                "msg": getXmlText(error, "m:message"),
+            }
+            self.add_warning(warnmsg % attrs)
+
     def check_html_w3 (self):
         """Check HTML syntax of this page (which is supposed to be HTML)
         with the online W3C HTML validator documented at
@@ -824,17 +865,14 @@ class UrlBase (object):
             u = urllib2.urlopen('http://validator.w3.org/check',
                 urllib.urlencode({
                     'fragment': self.get_content(),
-                    'output': 'xml',
+                    'output': 'soap12',
                 }))
             if u.headers.get('x-w3c-validator-status', 'Invalid') == 'Valid':
                 self.add_info(u"W3C Validator: %s" % _("valid HTML syntax"))
                 return
-            from xml.dom.minidom import parseString
-            dom = parseString(u.read())
-            elements = dom.getElementsByTagName('messages')[0].getElementsByTagName('msg')
-            for msg in [e.firstChild.wholeText for e in elements]:
-                self.add_warning(u"W3C HTML validation: %s" % msg)
+            self.check_w3_errors(u.read(), "W3C HTML")
         except Exception:
+            raise
             # catch _all_ exceptions since we dont want third party module
             # errors to propagate into this library
             err = str(sys.exc_info()[1])
@@ -868,11 +906,7 @@ class UrlBase (object):
             if r.getheader('X-W3C-Validator-Status', 'Invalid') == 'Valid':
                 self.add_info(u"W3C Validator: %s" % _("valid CSS syntax"))
                 return
-            from xml.dom.minidom import parseString
-            dom = parseString(r.read())
-            elements = dom.getElementsByTagName('m:errors')[0].getElementsByTagName('m:error')
-            for msg in [e.firstChild.wholeText for e in elements]:
-                self.add_warning(u"W3C HTML validation: %s" % msg)
+            self.check_w3_errors(r.read(), "W3C HTML")
         except Exception:
             # catch _all_ exceptions since we dont want third party module
             # errors to propagate into this library
@@ -932,26 +966,24 @@ class UrlBase (object):
     def parse_opera (self):
         """Parse an opera bookmark file."""
         log.debug(LOG_CHECK, "Parsing Opera bookmarks %s", self)
-        from ..bookmarks.opera import parse_bookmarks
-        for url, name, lineno in parse_bookmarks(self.get_content()):
+        from ..bookmarks.opera import parse_bookmark_data
+        for url, name, lineno in parse_bookmark_data(self.get_content()):
             url_data = get_url_from(url, self.recursion_level+1,
                self.aggregate, parent_url=self.url, line=lineno, name=name)
             self.aggregate.urlqueue.put(url_data)
 
     def parse_chromium (self):
-        """Parse a Google Chrome bookmark file."""
+        """Parse a Chromium or Google Chrome bookmark file."""
         log.debug(LOG_CHECK, "Parsing Chromium bookmarks %s", self)
-        from ..bookmarks.chromium import parse_bookmarks_data
-        for url, name in parse_bookmarks_data(self.get_content()):
+        from ..bookmarks.chromium import parse_bookmark_data
+        for url, name in parse_bookmark_data(self.get_content()):
             url_data = get_url_from(url, self.recursion_level+1,
                self.aggregate, parent_url=self.url, name=name)
             self.aggregate.urlqueue.put(url_data)
 
     def parse_text (self):
-        """
-        Parse a text file with on url per line; comment and blank
-        lines are ignored.
-        """
+        """Parse a text file with one url per line; comment and blank
+        lines are ignored."""
         log.debug(LOG_CHECK, "Parsing text %s", self)
         lineno = 0
         for line in self.get_content().splitlines():
