@@ -79,7 +79,7 @@ def get_modules_info ():
         from PyQt4 import QtCore
         lines.append(u"Qt: %s / PyQt: %s" %
                      (QtCore.QT_VERSION_STR, QtCore.PYQT_VERSION_STR))
-    except ImportError:
+    except (ImportError, AttributeError):
         pass
     # modules
     modules = [name for (mod, name) in Modules if fileutil.has_module(mod)]
@@ -179,6 +179,7 @@ class Configuration (dict):
         self['output'] = 'text'
         self['logger'] = None
         self["warningregex"] = None
+        self["warningregex_max"] = 5
         self["warnsizebytes"] = None
         self["nntpserver"] = os.environ.get("NNTP_SERVER", None)
         self["threads"] = 10
@@ -190,6 +191,7 @@ class Configuration (dict):
         self["checkcssw3"] = False
         self["scanvirus"] = False
         self["clamavconf"] = clamav.canonical_clamav_conf()
+        self["useragent"] = UserAgent
 
     def init_logging (self, status_logger, debug=None, handler=None):
         """
