@@ -17,6 +17,7 @@
 
 import os
 import sys
+import re
 import webbrowser
 from PyQt4 import QtCore, QtGui
 from .linkchecker_ui_main import Ui_MainWindow
@@ -108,6 +109,7 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.connect_widgets()
         self.init_config()
         self.read_config()
+        self.init_menu()
         self.init_app()
 
     def init_url (self, url):
@@ -117,6 +119,12 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.urlinput.setModel(self.recent)
         if url:
             self.urlinput.setText(url)
+        elif documents:
+            self.urlinput.setText(documents[0])
+
+    def init_menu (self):
+        """Add menu entries for bookmark file checking."""
+        self.urlinput.addMenuEntries(self.menuEdit)
 
     def init_app (self):
         """Set window size and position, GUI options and reset status."""
@@ -202,7 +210,7 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         try:
             self.config.read()
         except LinkCheckerError, msg:
-            self.config_error = msg
+            self.config_error = unicode(msg)
 
     def set_config (self):
         """Set configuration."""
@@ -215,6 +223,8 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
             self.config["threads"] = 1
         else:
             self.config.reset_loglevel()
+        if data["warningregex"]:
+            self.config["warningregex"] = re.compile(data["warningregex"])
 
     def get_status (self):
         """Return current application status."""
@@ -234,16 +244,16 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
             self.treeView.setSortingEnabled(True)
             self.treeView.scrollToTop()
             self.movie.stop()
+            # Reset progress information.
+            self.label_active.setText(u"0")
+            self.label_queued.setText(u"0")
+            self.label_checked.setText(u"0")
             self.label_busy.hide()
             self.menubar.setEnabled(True)
             self.urlinput.setEnabled(True)
         elif status == Status.checking:
             self.treeView.setSortingEnabled(False)
             self.debug.reset()
-            # Reset progress information.
-            self.label_active.setText(u"0")
-            self.label_queued.setText(u"0")
-            self.label_checked.setText(u"0")
             self.set_statusmsg(u"Checking site...")
             # disable commands
             self.menubar.setEnabled(False)
